/*
 * state_test.cpp
 *
 *  Created on: Sep 27, 2008
 *      Author: Santi Ontan
 */

#ifdef KITSCHY_DEBUG_MEMORY
#include "debug_memorymanager.h"
#endif

#ifdef _WIN32
#include "windows.h"
#endif

#include "stdio.h"
#include "math.h"
#include "stdlib.h"
#include "string.h"

#include "debug.h"

#include "GL/gl.h"
#include "SDL.h"
#include "SDL_mixer.h"

#include "BList.h"
#include "Symbol.h"
#include "2DCMC.h"
#include "auxiliar.h"
#include "GLTile.h"
#include "2DCMC.h"
#include "keyboardstate.h"
#include "randomc.h"
#include "VirtualController.h"

#include "Binterface.h"

#include "GLTManager.h"
#include "SoundManager.h"
#include "SFXManager.h"
#include "MusicManager.h"
#include "KObject.h"
#include "Knightmare.h"
#include "KnightmareApp.h"


bool m_show_panel = true;
bool g_preserve_angle = true;


int KnightmareApp::test_cycle(KEYBOARDSTATE *k)
{
	if (k->key_press(SDLK_ESCAPE)) return STATE_KONAMI;

	if (k->key_press(SDLK_TAB)) m_show_panel = (m_show_panel ? false:true);

//	if (k->key_pressed(SDLK_e)) m_game->map_advance(-1);
//	if (k->key_pressed(SDLK_d)) m_game->map_advance(1);

//	if (k->key_pressed(SDLK_PAGEUP)) m_game->map_advance(-20);
//	if (k->key_pressed(SDLK_PAGEDOWN)) m_game->map_advance(20);

	if (k->key_pressed(SDLK_q)) m_game->change_camera_angle(-0.5f);
	if (k->key_pressed(SDLK_a)) m_game->change_camera_angle(0.5f);

	if (k->key_pressed(SDLK_w)) m_game->change_zoom(1.01f);
	if (k->key_pressed(SDLK_s)) m_game->change_zoom(0.99f);
	if (k->key_pressed(SDLK_x)) m_game->set_zoom(1.0f);

	if (k->key_press(SDLK_z)) {
		g_preserve_angle = (g_preserve_angle ? false:true);
		m_game->setPreserveAngle(g_preserve_angle);
	}

	{
		VirtualController *vc;
		m_vc_l.Rewind();
		while(m_vc_l.Iterate(vc)) {
			vc->new_cycle();
			if (vc->m_ID==0) {
				if (k->key_pressed(m_keys_configuration1[KKEY_UP])) vc->m_joystick[VC_UP]=true;
				if (k->key_pressed(m_keys_configuration1[KKEY_DOWN])) vc->m_joystick[VC_DOWN]=true;
				if (k->key_pressed(m_keys_configuration1[KKEY_LEFT])) vc->m_joystick[VC_LEFT]=true;
				if (k->key_pressed(m_keys_configuration1[KKEY_RIGHT])) vc->m_joystick[VC_RIGHT]=true;
			}
			if (vc->m_ID==1) {
				if (k->key_pressed(m_keys_configuration2[KKEY_UP])) vc->m_joystick[VC_UP]=true;
				if (k->key_pressed(m_keys_configuration2[KKEY_DOWN])) vc->m_joystick[VC_DOWN]=true;
				if (k->key_pressed(m_keys_configuration2[KKEY_LEFT])) vc->m_joystick[VC_LEFT]=true;
				if (k->key_pressed(m_keys_configuration2[KKEY_RIGHT])) vc->m_joystick[VC_RIGHT]=true;
			}
		}
	}

	m_game->cycle(&m_vc_l,m_GLTM,m_SFXM,m_MusicM);

	return STATE_TEST;
}

void KnightmareApp::test_draw(void)
{
	char tmp[80];

	glClearColor(0, 0, 0, 1);
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT|GL_STENCIL_BUFFER_BIT);

    m_game->draw(m_GLTM);

    if (m_show_panel) {
		sprintf(tmp,"Level: %i",m_current_level);
		Binterface::print_left(tmp,m_font16,1,21,0,0,0,1);
		Binterface::print_left(tmp,m_font16,0,20,1,1,1,1);

		sprintf(tmp,"Map position (E/D, PAGE UP/DOWN): %i",m_game->get_map_position());
		Binterface::print_left(tmp,m_font16,1,41,0,0,0,1);
		Binterface::print_left(tmp,m_font16,0,40,1,1,1,1);

		sprintf(tmp,"Zoom: %g (W/S/X)",m_game->get_zoom());
		Binterface::print_left(tmp,m_font16,1,81,0,0,0,1);
		Binterface::print_left(tmp,m_font16,0,80,1,1,1,1);

		sprintf(tmp,"Angle: %g (Q/A)",m_game->get_camera_angle());
		Binterface::print_left(tmp,m_font16,1,101,0,0,0,1);
		Binterface::print_left(tmp,m_font16,0,100,1,1,1,1);

		sprintf(tmp,"Preserve Angle: %s (Z)",(g_preserve_angle ? "true":"false"));
		Binterface::print_left(tmp,m_font16,1,121,0,0,0,1);
		Binterface::print_left(tmp,m_font16,0,120,1,1,1,1);
    } // if

}

